package Weblog;

BEGIN {
    use strict;
    use Carp;
    use Weblog::Config;
    use Weblog::Channel;
    use Weblog::Output;
    use Weblog::Item;
    use Weblog::Auth;
    use vars qw($VERSION $AUTOLOAD @ISA);
    $VERSION = '0.1';
    @ISA = qw(Weblog::Channel Weblog::Output Weblog::Auth);
}

# list of valid class attributes
my %ok_fields = (
		 config          => undef,
		 channels        => [],
		 current_channel => undef,
		 config_file     => undef
	       );

sub new {
#**********************************************************************#
    # determine class name
    my $pkg = shift;
    my $class = ref($pkg) || $pkg;
    
    # build new instance of object
    my $self = bless 
    {
        _fields => \%ok_fields
        }, $class;
    
    # initalize variables in object if values were passed
    $self->_initialize(@_);
    
    # return pointer to the new object
    return $self;
#**********************************************************************#
}

sub _initialize {
#**********************************************************************#
    my $self = shift;

    # Copy the hashes instead of using them directly to avoid
    # problems with multiple Weblog objects being used concurrently
    my %template=%ok_fields;
    $self->{_fields} = \%template;

    # initialize class attributes
    $self->config_file(shift);

    # load configuration parameters if the user passed the
    # config.xml file location when they created a new
    # instance of the class.
    if ($self->config_file) {
	$self->config(new Weblog::Config($self->config_file));
    } else {
	croak ("Must specify a Weblog configuration file");
    }
#**********************************************************************#
}

sub AUTOLOAD {
#**********************************************************************#
    my $self = shift;
    my $type = ref($self) || croak "$self is not an object\n";
    my $name = $AUTOLOAD;
    $name =~ s/.*://;
    unless (exists $self->{_fields}->{$name}) {
        croak "Unregistered entity: Can't access $name field in object of class $type";
    }
    if (@_) {
        return $self->{_fields}->{$name} = shift;
    } else {
	return $self->{_fields}->{$name};
    }
#**********************************************************************#
}

sub DESTROY {
#**********************************************************************#
    my $self = shift;
    $self = undef;
#**********************************************************************#
}

########################################################################
# METHOD PROTOTYPES                                                    #
########################################################################
# sub list_channels ({%options}) { }            

########################################################################
# CLASS METHODS                                                        #
########################################################################

sub add_item {
#**********************************************************************#
# SYNTAX: &add_item($item, @channels);                                 #
# PARAMS: $item - reference to a Weblog::Item object                   #
#         @channels - array of strings containing the names of the     #
#                     channels that the item should be added to        #
# RETURN: 1 - item added successfully                                  #
#         0 - Could not add item                                       #
#**********************************************************************#
    my ($self, $item, $mode, @channels) = @_;
    
    # check incoming data
    if (!($item =~ /Weblog::Item/)) {
	warn ("Must pass a valid Weblog::Item object to Weblog::add_item");
	return 0;
    }
    if (!($mode eq 'insert' || $mode eq 'append')) {
	warn ("Must pass valid add mode. Either insert or append");
	return 0;
    }
    
    # insert the item in each specified channel
    foreach my $channel (@channels) {
	my $channelobj = new Weblog::Channel($self,$channel);
	$channelobj->add_item($item,$mode);
	$channelobj->save;
    }

    return 1;
#**********************************************************************#
# END SUBROUTINE: &add_item( $item, @channels);                        #
#**********************************************************************#
}

sub list_channels {
#**********************************************************************#
# SYNTAX: &list_channels({%options});                                  #
# PARAMS: %options = (                                                 #
#                     local  => ([1] | 0),                             #
#                     remote => (1 | [0]),                             #
#                     user   => username,                              #
#                     passwd => password,                              #
#                     server => 'xmlrpc.com:8080/cgi-bin/xmlrpc'       #
#                    );                                                #
# RETURN: @channels({name=>,title=>,description=>,...) -list of        #
#         available channels on local and remote or remote server.     #
#         0 - Could not list channels                                  #
#**********************************************************************#
    # set method parameters
    my ($self,%options) = @_;
    
    # initialize local parameters
    my @channels = ();

    # set defaults
    $options{'local'} = 1 unless exists $options{'local'};
    $options{'remote'} = 0 unless exists $options{'remote'};

    # build channel array from config.xml file based on what 
    # whether the user wants local and/or remote channels listed
    foreach my $channel_name (keys(%{$self->config->channel})) {
	my $channel = $self->config->channel->{$channel_name};
	if ($options{'remote'} && $channel->{'server'}) {
	    push (@channels,$channel);
	} elsif($options{'local'} && !($channel->{'server'})) {
	    push (@channels,$channel);
	}
    }

    # retrieve remote channel list if a username and
    if ($options{'name'} && $options{'passwd'} && $options{'server'}) {
	
    }

    return @channels;
#**********************************************************************#
# END SUBROUTINE: &list_channels({%options});                          #
#**********************************************************************#
}






1;
