package Weblog::Client;

########################################################################
# LOAD MODULES                                                         #
########################################################################
BEGIN {
    use strict;
    use Carp;
    use Frontier::Client;
    use vars qw($VERSION $AUTOLOAD @ISA);
    $VERSION = '0.1';
    @ISA = qw();
}

########################################################################
# CLASS INITIALIZATION                                                 #
########################################################################
# list of valid class attributes
my %ok_fields = (
		 'client' => undef,
		 'url'    => undef
	       );

sub new {
#**********************************************************************#
    # determine class name
    my $pkg = shift;
    my $class = ref($pkg) || $pkg;
    
    # build new instance of object
    my $self = bless 
    {
        _fields => \%ok_fields
        }, $class;
    
    # initalize variables in object if values were passed
    $self->_initialize(@_);
    
    # return pointer to the new object
    return $self;
#**********************************************************************#
}

sub _initialize {
#**********************************************************************#
    my $self = shift;
    my %hash = @_;
    foreach my $key (keys(%hash)) {
	$self->$key($hash{$key});
    }

    $self->client(new Frontier::Client(url => $self->url, debug => 1));
#**********************************************************************#
}

sub AUTOLOAD {
#**********************************************************************#
    my $self = shift;
    my $type = ref($self) || croak "$self is not an object\n";
    my $name = $AUTOLOAD;
    $name =~ s/.*://;
    unless (exists $self->{_fields}->{$name}) {
        croak "Unregistered entity: Can't access $name field in object of class $type";
    }
    if (@_) {
        return $self->{$name} = shift;
    } else {
	return $self->{$name};
    }
#**********************************************************************#
}

sub DESTROY {
#**********************************************************************#
    my $self = shift;
    $self = undef;
#**********************************************************************#
}

sub list_channels {
#**********************************************************************#
# SYNTAX: &list_channels;                                              #
# PARAMS:                                                              #
# RETURN:                                                              #
#**********************************************************************#
    my $self = shift;
    return $self->client->call("list_channels");
#**********************************************************************#
# END SUBROUTINE: &list_channels;                                      #
#**********************************************************************#
}

sub retrieve_channel {
    my $self = shift;
    my $channel_name = shift;
    return $self->client->call("retrieve_channel",$channel_name);
}

1;
