package Weblog::Config;

########################################################################
# LOAD MODULES                                                         #
########################################################################
BEGIN {
    use strict;
    use Carp;
    use XML::Simple;
    use vars qw($VERSION $AUTOLOAD @ISA);
    $VERSION = '0.1';
    @ISA = qw();
}

########################################################################
# CLASS INITIALIZATION                                                 #
########################################################################
# list of valid class attributes
my %ok_fields = (
		 'script_url'      => undef,
		 'default_channel' => undef,
		 'language'        => undef,
		 'security'        => undef,
		 'channel'         => undef,
		 'lock_retries'    => undef,
		 'archive_dir'     => undef,
		 'temp_dir'        => undef,
		 'lock_dir'        => undef,
		 'templates'       => undef,
		 'file'            => undef,
		 'xhoo_root'       => undef,
		 'xhoo_url'        => undef,
		 'xhoo_output'     => undef
	       );

sub new {
#**********************************************************************#
    # determine class name
    my $pkg = shift;
    my $class = ref($pkg) || $pkg;
    
    # build new instance of object
    my $self = bless 
    {
        _fields => \%ok_fields
        }, $class;
    
    # initalize variables in object if values were passed
    $self->_initialize(@_);
    
    # return pointer to the new object
    return $self;
#**********************************************************************#
}

sub _initialize {
#**********************************************************************#
    my $self = shift;
    if (@_ && -e $_[0]) {
	$self->file($_[0]);
	$self->load_config;
    }
#**********************************************************************#
}

sub AUTOLOAD {
#**********************************************************************#
    my $self = shift;
    my $type = ref($self) || croak "$self is not an object\n";
    my $name = $AUTOLOAD;
    $name =~ s/.*://;
    unless (exists $self->{_fields}->{$name}) {
        croak "Unregistered entity: Can't access $name field in object of class $type";
    }
    if (@_) {
        return $self->{_fields}->{$name} = shift;
    } else {
	return $self->{_fields}->{$name};
    }
#**********************************************************************#
}

sub DESTROY {
#**********************************************************************#
    my $self = shift;
    $self = undef;
#**********************************************************************#
}

sub load_config {
#**********************************************************************#
# SYNTAX:         #
# PARAMS:   #
#           #
# RETURN: 1 -          #
#         0 -          #
#**********************************************************************#
    my $self = shift;
    
    # get the config file name
    my $file;
    if (@_) {
	$file = shift;
    } elsif($self->file) {
	$file = $self->file;
    } else {
	croak ("You must specify a configuration file to load");
    }

    # load the configuration file using the XML::Simple module
    my $config = XMLin($file, keyattr=>["channel"], forcearray => ["channel"]);
    
    # load up channels hash
    my %channels = ();
    for my $channel ( @{$config->{channel}}) {
	$channels{$channel->{name}} = $channel;
    }
    $config->{'channel'} = \%channels;

    # load up the local object attributes
    foreach my $field (keys(%ok_fields)) {
	$self->$field($config->{$field}) if $config->{$field};
    }
#**********************************************************************#
# END SUBROUTINE &load_config ($xmlfile);                              #
#**********************************************************************#
}

1;
