package Weblog::Item;

########################################################################
# LOAD MODULES                                                         #
########################################################################
BEGIN {
    use strict;
    use Carp;
    use HTML::Entities;
    use vars qw($VERSION $AUTOLOAD @ISA);
    $VERSION = '0.1';
    @ISA = qw();
}

########################################################################
# CLASS INITIALIZATION                                                 #
########################################################################
# list of valid class attributes
my %ok_fields = (
		 'title'       => undef,
		 'link'        => undef,
		 'description' => undef,
		 'date'        => undef,
		 'creator'     => undef,
		 'source'      => undef,
		 'format'      => undef,
		 'identifier'  => undef,
		 'type'        => undef,
		 'subject'     => undef,
		 'dc'          => undef,
		 'item'        => undef,
		 'channel'     => undef,
		 'taxo'        => undef,
		 '//purl.org/dc/elements/1.1/' => undef
	       );

sub new {
#**********************************************************************#
    # determine class name
    my $pkg = shift;
    my $class = ref($pkg) || $pkg;
    
    # build new instance of object
    my $self = bless 
    {
        _fields => \%ok_fields
        }, $class;
    
    # initalize variables in object if values were passed
    $self->_initialize(@_);
    
    # return pointer to the new object
    return $self;
#**********************************************************************#
}

sub _initialize {
#**********************************************************************#
    my $self = shift;
    my %hash = @_;

    # Copy the hashes instead of using them directly to avoid
    # problems with multiple Weblog::Item objects being used concurrently
    my %template=%ok_fields;
    $self->{_fields} = \%template;         

    # assign object attribute values
    foreach my $key (keys(%hash)) {
	$self->$key($hash{$key});
    }
    
    # convert DCMI has values into object attributes
    if ($self->dc) {
	my $dc = $self->dc;
	foreach my $item (keys(%$dc)) {
	    $self->$item($dc->{$item}) if exists $self->{_fields}->{$item};
	}
    }

    # now get rid of our extraneous data that was passed in;
    $self->item(undef);
    $self->{_fields}->{'//purl.org/dc/elements/1.1/'} = undef;
    $self->dc(undef);

    # assign item identifier
    $self->identifier(&get_identifier) unless $self->identifier;
#**********************************************************************#
}

sub AUTOLOAD {
#**********************************************************************#
    my $self = shift;
    my $type = ref($self) || croak "$self is not an object\n";
    my $name = $AUTOLOAD;
    $name =~ s/.*://;
    unless (exists $self->{_fields}->{$name}) {
        croak "Unregistered entity: Can't access $name field in object of class $type";
    }
    if (@_) {
        return $self->{_fields}->{$name} = shift;
    } else {
	return $self->{_fields}->{$name};
    }
#**********************************************************************#
}

sub DESTROY {
#**********************************************************************#
    my $self = shift;
    $self = undef;
#**********************************************************************#
}

########################################################################
# METHOD PROTOTYPES                                                    #
########################################################################
# sub list_channels ({%options}) { }                     

########################################################################
# CLASS METHODS                                                        #
########################################################################

sub get_identifier {
#**********************************************************************#
# SYNTAX: &get_identifier;                                             #
# PARAMS:                                                              #
# RETURN:                                                              #
#**********************************************************************#
    return time.":".int(rand(999999999999));
#**********************************************************************#
# END SUBROUTINE: &load_channel;                                       #
#**********************************************************************#
}


1;
