package Weblog::Output;

########################################################################
# LOAD MODULES                                                         #
########################################################################
BEGIN {
    use strict;
    use Carp;
    use Weblog::Output::Template;
    use vars qw($VERSION $AUTOLOAD @ISA);
    $VERSION = '0.1';
    @ISA = qw();
}

########################################################################
# CLASS INITIALIZATION                                                 #
########################################################################
# list of valid class attributes
my %ok_fields = (
		 'types' => {
		     'Template' => 1
		 },
		 'output_type' => undef,
		 'channel'     => undef
	       );

sub new {
#**********************************************************************#
    # determine class name
    my $pkg = shift;
    my $class = ref($pkg) || $pkg;
    
    # build new instance of object
    my $self = bless 
    {
        _fields => \%ok_fields
        }, $class;
    
    # initalize variables in object if values were passed
    $self->_initialize(@_);
    
    # return pointer to the new object
    return $self;
#**********************************************************************#
}

sub _initialize {
#**********************************************************************#
    my $self = shift;

    # Copy the hashes instead of using them directly to avoid
    # problems with multiple objects being used concurrently
    my %template=%ok_fields;
    $self->{_fields} = \%template;
    
    # assign values to local variables
    my ($channel,$type) = @_;

    # ensure that a channel object and type were passed
    croak ("Must pass Weblog::Channel object and a valid output type")
	unless (($channel =~ /Weblog::Channel/) 
		&& exists($self->{_fields}->{'types'}->{$type}));

    # set the output format
    $self->output_type($type);
    $self->channel($channel);
#**********************************************************************#
}

sub AUTOLOAD {
#**********************************************************************#
    my $self = shift;
    my $type = ref($self) || croak "$self is not an object\n";
    my $name = $AUTOLOAD;
    $name =~ s/.*://;
    unless (exists $self->{_fields}->{$name}) {
        croak "Unregistered entity: Can't access $name field in object of class $type";
    }
    if (@_) {
        return $self->{$name} = shift;
    } else {
	return $self->{$name};
    }
#**********************************************************************#
}

sub DESTROY {
#**********************************************************************#
    my $self = shift;
    $self = undef;
#**********************************************************************#
}

########################################################################
# METHOD PROTOTYPES                                                    #
########################################################################
# sub output { }


########################################################################
# CLASS METHODS                                                        #
########################################################################

sub output {
#**********************************************************************#
# SYNTAX: &output ($format);                                           #
# PARAMS: $format - text containing the format type, i.e. html, wml    #
# RETURN:                                                              #
#**********************************************************************#
    my ($self,$format) = @_;
    my $output;
    if ($self->output_type eq 'Template') {
	$output = new Weblog::Output::Template($self->channel);
    }
    
    $output->output($format);
#**********************************************************************#
# END SUBROUTINE: &output ($format);                                   #
#**********************************************************************#
}

sub outputall {
#**********************************************************************#
# SYNTAX: &outputall;                                                  #
# PARAMS:                                                              #
# RETURN:                                                              #
#**********************************************************************#
    my $self = shift;
    my $output;
    if ($self->output_type eq 'Template') {
	$output = new Weblog::Output::Template($self->channel);
    }
    
    $output->outputall;
#**********************************************************************#
# END SUBROUTINE: &output;                                             #
#**********************************************************************#
}


1;

