package Weblog::Server;

########################################################################
# LOAD MODULES                                                         #
########################################################################
BEGIN {
    use strict;
    use Carp;
    use Frontier::Daemon;
    use Frontier::RPC2;
    use MIME::Base64;
    use POSIX 'setsid';
    use Weblog;
    use vars qw($VERSION $AUTOLOAD @ISA);
    $VERSION = '0.1';
    @ISA = qw(Frontier::Daemon);
}

########################################################################
# CLASS INITIALIZATION                                                 #
########################################################################
# list of valid class attributes
my %ok_fields = (
		 'channels'    => undef,
		 'port'        => undef,
		 'config_file' => undef,
		 'weblog'      => undef,
		 'server'      => undef
	       );

sub new {
#**********************************************************************#
    # determine class name
    my $pkg = shift;
    my $class = ref($pkg) || $pkg;
    
    # build new instance of object
    my $self = bless 
    {
        _fields => \%ok_fields, 
        %ok_fields
        }, $class;
    
    # initalize variables in object if values were passed
    $self->_initialize(@_);
    
    # return pointer to the new object
    return $self;
#**********************************************************************#
}

sub _initialize {
#**********************************************************************#
    my $self = shift;
    my %hash = @_;
    foreach my $key (keys(%hash)) {
	$self->$key($hash{$key});
    }

    # verify that we have the port, list of channels, and config file
    croak ("Must define a port that the Weblog daemon will run on")
	unless $self->port;
    croak ("Must define a valid Weblog XML configuration file")
	unless -e $self->config_file;

    # create an instance of the Weblog class
    $self->weblog(new Weblog($self->config_file));

    # validate the list of channels passed to the daemon
    my @channels = $self->weblog->list_channels(local => 1);
    foreach my $pchannel (@{$self->channels}) {
	my $listed = 0;
	foreach my $lchannel (@channels) {
	    $listed = 1 if $lchannel->{'name'} eq $pchannel;
	}
	croak ("$pchannel is not a valid channel")
	    unless $listed == 1;
    }

    # turn the server into a demon.
    #&daemonize;

    # initialize the Weblog Server
    $self->server(new Frontier::Daemon (
    LocalPort => $self->port,
    methods   => {
	'list_channels'   => \&list_channels($self)
    }));             
#**********************************************************************#
}

sub AUTOLOAD {
#**********************************************************************#
    my $self = shift;
    my $type = ref($self) || croak "$self is not an object\n";
    my $name = $AUTOLOAD;
    $name =~ s/.*://;
    unless (exists $self->{_fields}->{$name}) {
        croak "Unregistered entity: Can't access $name field in object of class $type";
    }
    if (@_) {
        return $self->{$name} = shift;
    } else {
	return $self->{$name};
    }
#**********************************************************************#
}

sub DESTROY {
#**********************************************************************#
    my $self = shift;
    $self = undef;
#**********************************************************************#
}


sub daemonize {
#**********************************************************************#
# SYNTAX: &daemonize;                                                  #
# PARAMS:                                                              #
# RETURN:                                                              #
#**********************************************************************#
    chdir '/' or die "Can't chdir to /: $!";
    #open STDIN, '/dev/null' or die "Can't read /dev/null: $!";
    #open STDOUT, '>/dev/null' or die "Can't write to /dev/null: $!";
    defined(my $pid = fork) or die "Can't fork: $!";
    exit if $pid;
    setsid or die "Can't start a new session: $!";
    #open STDERR, '>&STDOUT' or die "Can't dup stdout: $!";
#**********************************************************************#
# END SUBROUTINE: &daemonize;                                          #
#**********************************************************************#
}       

sub list_channels {
#**********************************************************************#
# SYNTAX: &list_channels;                                              #
# PARAMS:                                                              #
# RETURN:                                                              #
#**********************************************************************#
    my $self = shift;
    my @channels = $self->weblog->list_channels;
    my @lchannels;
    foreach my $channel (@channels) {
	push(@lchannels,$channel->{'name'});
	print "ha";
    }
    
    return [@lchannels];
#**********************************************************************#
# END SUBROUTINE: &list_channels;                                      #
#**********************************************************************#
}       

1;
