package Xhoo;

BEGIN {
    use strict;
    use Carp;
    use XML::RSS;
    use HTML::Entities;
    use POSIX qw(strftime);
    #use Data::Dumper;
    use vars qw($VERSION $AUTOLOAD @ISA);
    $VERSION = '0.1';
    @ISA = qw();
}

# list of valid class attributes
my %ok_fields = (
		 'baseurl'     => undef,
		 'rootdir'     => undef,
		 'categories'  => undef,
		 'output' => undef
	       );

sub new {
#**********************************************************************#
    # determine class name
    my $pkg = shift;
    my $class = ref($pkg) || $pkg;
    
    # build new instance of object
    my $self = bless 
    {
        _fields => \%ok_fields
        }, $class;
    
    # initalize variables in object if values were passed
    $self->_initialize(@_);
    
    # return pointer to the new object
    return $self;
#**********************************************************************#
}

sub _initialize {
#**********************************************************************#
    my $self = shift;
    my %hash = @_;

    # Copy the hashes instead of using them directly to avoid
    # problems with multiple Weblog objects being used concurrently
    my %template=%ok_fields;
    $self->{_fields} = \%template;

    # assign object attribute values
    foreach my $key (keys(%hash)) {
	$self->$key($hash{$key});
    }
    
    # build a list of the cateogories for the directory
    $self->list_categories($self->rootdir);
#**********************************************************************#
}

sub AUTOLOAD {
#**********************************************************************#
    my $self = shift;
    my $type = ref($self) || croak "$self is not an object\n";
    my $name = $AUTOLOAD;
    $name =~ s/.*://;
    unless (exists $self->{_fields}->{$name}) {
        croak "Unregistered entity: Can't access $name field in object of class $type";
    }
    if (@_) {
        return $self->{_fields}->{$name} = shift;
    } else {
	return $self->{_fields}->{$name};
    }
#**********************************************************************#
}

sub DESTROY {
#**********************************************************************#
    my $self = shift;
    $self = undef;
#**********************************************************************#
}

########################################################################
# METHOD PROTOTYPES                                                    #
########################################################################        

########################################################################
# CLASS METHODS                                                        #
########################################################################
sub add_link {
#**********************************************************************#
# SYNTAX:                                  #
# PARAMS:                  #
# RETURN: 1 -                                   #
#         0 -                                    #
#**********************************************************************#
    # grab the new link parameters
    my ($self, $dir, $datafile, $identifier, %link) = @_;
    $dir = "/$dir" unless $dir =~ /^\//;
    my $file = $self->rootdir . $dir."/".$datafile;
    
    ###############
    # Check Input #
    ###############
    # make sure the directory and datafile name were passed
    croak ("Must pass valid category and datafile name")
	unless (-d $self->rootdir.$dir);

    # make sure we have all required fields to add a new link to the directory
    croak ("url, title, description, name, and email are required fields")
	unless (exists($link{'url'}) && exists($link{'title'}) && exists($link{'name'})
		&& exists($link{'description'}) && exists($link{'email'}));

    # get a date/time stamp
    my $date = $self->date_time_stamp();
  
    #######################
    # Open directory file #
    #######################
    my $rss = new XML::RSS (version => '1.0');
    if (-e $file) {
	$rss->parsefile($file) || croak("Cannot open $file for write: $!");
    }
  
    # update the channel info
    $rss->channel(
		  'title' => "Xhoo: $dir",
		  'link'  => $self->baseurl . $dir
		  );

    ######################
    # add or update item #
    ######################
    # if the identifier that was passed to the method matches
    # an existing identifier, edit the item instead
    foreach my $item (@{$rss->items}) {
	if ($item->{'dc'}->{'identifier'} eq $identifier) {
	    #warn ("editing Xhoo item instead of adding");
	    $self->edit_link($dir,$datafile,$identifier,%link);
	    return;
	}
    }

    # add the item to the XML::RSS object
    $rss->add_item(title       => $self->encode($link{'title'}),
		   link        => $self->encode($link{'url'}),
		   description => $self->encode($link{'description'}),
		   dc => {
		       creator    => $self->encode($link{'name'}),
		       date       => $date,
		       identifier => $identifier,
		       source     => $self->encode($link{'email'})
		   },
                       mode  => 'append'
                       ) || croak ("Cannot add item to directory channel");      

    # update the date stamp on the directory channel
    $rss->channel(dc => { 'date' => $date });

    # lock the directory channel
    $self->lock_datafile($file) || croak "Cannot lock $file";

    # save the RSS object to the disk
    $rss->save($file);
    
    # output the html file
    $self->output_html($dir,$rss);

    $self->unlock_datafile($file);
#**********************************************************************#
# END SUBROUTINE:                         #
#**********************************************************************#
}

sub date_time_stamp {
#**********************************************************************#
# SYNTAX:         #
# PARAMS:   #
#           #
# RETURN: 1 -          #
#         0 -          #
#**********************************************************************#
    return strftime("%Y-%m-%dT%H:%M",gmtime);
#**********************************************************************#
# END SUBROUTINE                        #
#**********************************************************************#
}

sub delete_link {
#**********************************************************************#
# SYNTAX:                                  #
# PARAMS:                  #
# RETURN: 1 -                                   #
#         0 -                                    #
#**********************************************************************#
    # grab the new link parameters
    my ($self, $dir, $datafile, $identifier) = @_;    
    $dir = "/$dir" unless $dir =~ /^\//;
    my $file = $self->rootdir . $dir."/".$datafile;

    # init local variables
    my $date;
    my $pos = 0;
  
    # make sure the directory and datafile name were passed
    #croak ("Must pass valid category, datafile, and identifier")
    return unless (-e "$file" && $identifier);

    # get a date/time stamp
    $date = $self->date_time_stamp();
    
    # load the rss file
    my $rss = new XML::RSS (version => '1.0');
    $rss->parsefile($file) || croak("Cannot open $file for write: $!");

    # find the position of the item we want to edit
    for(my $counter=0; $counter < @{$rss->{'items'}}; $counter++) {
	my $item = $rss->{'items'}->[$counter];
	if ($item->{'dc'}->{'identifier'} eq $identifier) {
	    $pos = $counter;
	    last;
	}
    }

    # delete the item
    splice(@{$rss->{'items'}},$pos,1) || croak ("Cannot delete item from directory channel");

    # lock the directory channel
    $self->lock_datafile($file);

    # update the date stamp on the directory channel
    $rss->channel(dc => { 'date' => $date });

    # save the RSS object to the disk
    $rss->save($file);
    
    # output the html file
    $self->output_html($dir,$rss);

    $self->unlock_datafile($file);
#**********************************************************************#
# END SUBROUTINE:                         #
#**********************************************************************#
}

sub edit_link {
#**********************************************************************#
# SYNTAX:                                  #
# PARAMS:                  #
# RETURN: 1 -                                   #
#         0 -                                    #
#**********************************************************************#
    # grab the new link parameters
    my ($self, $dir, $datafile, $identifier, %link) = @_;
    $dir = "/$dir" unless $dir =~ /^\//;
    my $file = $self->rootdir . $dir."/".$datafile;

    # init local variables
    my $date;
    my $pos = 0;

    # make sure the directory and datafile name were passed
    croak ("Must pass valid directory and datafile name")
	unless (-e "$file");

    # make sure we have all required fields to add a new link to the directory
    croak ("url, title, description, name, and email are required fields")
	unless (exists($link{'url'}) && exists($link{'title'}) && exists($link{'name'})
		&& exists($link{'description'}) && exists($link{'email'}));

    # get a date/time stamp
    $date = $self->date_time_stamp();
    
    # load the rss file
    my $rss = new XML::RSS (version => '1.0');
    $rss->parsefile($file) || croak("Cannot open $file for write: $!");

    # find the position of the item we want to edit
    for(my $counter=0; $counter < @{$rss->{'items'}}; $counter++) {
	my $item = $rss->{'items'}->[$counter];
	if ($item->{'dc'}->{'identifier'} eq $identifier) {
	    $pos = $counter;
	    last;
	}
    }

    # update the item data
    $rss->{'items'}->[int($pos)]->{'title'} = $self->encode($link{'title'});
    $rss->{'items'}->[int($pos)]->{'link'} = $self->encode($link{'url'});
    $rss->{'items'}->[int($pos)]->{'description'} = $self->encode($link{'description'});
    $rss->{'items'}->[int($pos)]->{'dc'}->{'creator'} = $self->encode($link{'name'});
    $rss->{'items'}->[int($pos)]->{'dc'}->{'date'} = $date;
    $rss->{'items'}->[int($pos)]->{'dc'}->{'source'} = $self->encode($link{'email'});

    # lock the directory channel
    $self->lock_datafile($file) || croak "Cannot lock $file";

    # update the date stamp on the directory channel
    $rss->channel(dc => { 'date' => $date });

    # save the RSS object to the disk
    $rss->save($file);

    # output the html file
    $self->output_html($dir,$rss);

    $self->unlock_datafile($file);
#**********************************************************************#
# END SUBROUTINE:                         #
#**********************************************************************#
}

sub encode {
#**********************************************************************#
# SYNTAX: &encode($text);                                              #
# PARAMS: $text - the text that will be encoded                        #
# RETURN: $text - the encoded string                                   #
#**********************************************************************#
    my $self = shift;
    my $text = shift;
    return 0 unless $text;
    $text = HTML::Entities::decode($text);
    $text =~s/\r//gsi;        # delete carraige returns
    $text =~s/\s*\n\s*/ /gsi; # replace returns w/ space
    $text =~ s/[^\w\s\\\/\<\>\&\@\.\,\:\;\'\"\{\}\[\]\-\_\+\=\(\)\!\~\`\@\#\$\%\^\*\?]//g;
    $text = HTML::Entities::encode($text, '\200-\377\'\%\&\<\>\"');
    return $text;
#**********************************************************************#
# END SUBROUTINE &encode($text)                                        #
#**********************************************************************#
}

sub get_identifier {
#**********************************************************************#
# SYNTAX: &get_identifier;                                             #
# PARAMS:                                                              #
# RETURN:                                                              #
#**********************************************************************#
    return time.":".int(rand(999999999999));
#**********************************************************************#
# END SUBROUTINE: &load_channel;                                       #
#**********************************************************************#
}

sub list_categories {
#**********************************************************************#
# SYNTAX:                                  #
# PARAMS:                  #
# RETURN: 1 -                                   #
#         0 -                                    #
#**********************************************************************#
    my ($self,$dir) = @_;
    my $reldir = $dir;
    my $rootdir = $self->rootdir;
    $reldir =~ s#$rootdir/?##;
    $reldir = "/" if $reldir !~ /\S+/;
    $reldir = "/".$reldir unless $reldir =~ /^\//;
    push(@{$self->{_fields}->{'categories'}},$reldir);
    #warn($reldir);

    opendir DIR, $dir || die "Cannot open $dir: $!\n";
    my @dirs = grep -d, map "$dir/$_", grep !/^\./, readdir DIR;
    closedir DIR;

    foreach my $newdir (@dirs) {   
	&list_categories($self,$newdir);
    }    
#**********************************************************************#
# END SUBROUTINE:                         #
#**********************************************************************#
}

sub list_links {
#**********************************************************************#
# SYNTAX:                                  #
# PARAMS:                  #
# RETURN: 1 -                                   #
#         0 -                                    #
#**********************************************************************#
    my $datafile = shift;
    my @entries = ();
    if (-e $datafile) {
	open(DATA,$datafile) || die "Cannot open $datafile: $!";
	my $csv = Text::CSV_XS->new();
	while (<DATA>) {
	    chomp;
	    $csv->parse($_);
	    my @columns = $csv->fields();
	    push(@entries,\@columns);
	}
    }
    return \@entries;
#**********************************************************************#
# END SUBROUTINE:                         #
#**********************************************************************#
}

sub lock_datafile {
#**********************************************************************#
# SYNTAX:                                  #
# PARAMS:                  #
# RETURN: 1 -                                   #
#         0 -                                    #
#**********************************************************************#
    # grab the new link parameters
    my ($self,$file) = @_;
  
    # lock semaphore file
    open(SEM, "> $file.semaphore") || croak "Cannot open $file.semaphore for write: $!";
    flock(SEM, 2) || croak "Cannot get exclusive lock for $file.semaphore: $!";
#**********************************************************************#
# END SUBROUTINE:                         #
#**********************************************************************#
}

sub output_html {
#**********************************************************************#
# SYNTAX:                                  #
# PARAMS:                  #
# RETURN: 1 -                                   #
#         0 -                                    #
#**********************************************************************#
    my ($self,$category,$rss) = @_;

    # figure out what category we're adding the item to
    my $rootdir = $self->rootdir;
    my $dir = "$rootdir$category";
    #$category =~ s#$rootdir/?##;

    #########################################
    # figure out what Xhoo template to use. # 
    #########################################
    # If there's a local 
    # template use it, otherwise use the global template
    my $template_file;
    if (-e "$dir/xhoo.tmpl") {
	$template_file = "$dir/xhoo.tmpl";
    } else {
	$template_file = $self->rootdir . "/../templates/xhoo.tmpl";
    }

    ##############################
    # figure out output filename #
    ##############################
    my $file = "$dir/" . $self->output;

    #warn("$template_file : $file");
    ##################################
    # Build directory location array #
    ##################################
    my @tmpl_location = ();
    my @parts = split(/\//,$category);

    for (my $i=0; $i < @parts; $i++) {
	my %row;
	$row{'title'} = $parts[$i];
	$row{'link'} = $self->baseurl . join('/',@parts[0..$i]) . "/" . $self->output; 
	push(@tmpl_location,\%row);
    }
    
    ############################
    # Build list of categories #
    ############################
    my @tmpl_categories = ();
    my $mydir = $dir;
    $mydir =~ s/\/+$//;

    opendir DIR,$mydir ||  &error("Cannot open $mydir: $!");
    my @dirs = sort(grep -d, map "$mydir/$_", grep !/^\./, readdir DIR);
    closedir DIR;
    
    foreach my $thisdir (@dirs) {
	my %row = ();
	#warn
	$thisdir =~ s/$rootdir$category//;
	$thisdir =~ s/\/+$//g;
	$thisdir =~ s/^\/+//g;

	my $url;
	if ($category eq '/') {
	    $url = $self->baseurl . "$thisdir";
	} else {
	    $url = $self->baseurl . "$category/$thisdir";
	}

	my $pdir = $thisdir;
	$pdir =~ s/_/ /g;

	$row{'link'} = $url;
	$row{'title'} = $pdir;
	push(@tmpl_categories,\%row);
    }

    ##################################
    # Buildlist directory links list #
    ##################################
    my @tmpl_links = ();
    my $count = 1;

    # loop over each RSS item
    foreach my $item (@{$rss->{'items'}}) {
	my %row = ();
	# skip the item unless there's a title and link
	next unless $item->{'title'} && $item->{'link'};
	
	$row{'title'} = $item->{'title'};
	$row{'link'}  = $item->{'link'};
	$row{'description'} = $item->{'description'};
      
	push(@tmpl_links,\%row);
	$count++;
    }

    ###################################
    # Assign values to HTML::Template #
    ###################################
    # create a new instance of the HTML::Template module 
    my $template = HTML::Template->new(filename          => $template_file,
				       die_on_bad_params => 0);

    # set the arrays in template
    $template->param(RSS_LOOP        => \@tmpl_links);
    $template->param(LOCATION_LOOP   => \@tmpl_location);
    $template->param(CATEGORIES_LOOP => \@tmpl_categories);
    $template->param(FULLNAME        => $rss->channel('title'));
    $template->param(XHOO_URL        => $self->baseurl);
    
    ##########################
    # Create the output file #
    ##########################
    # open the format output file for write or croak
    open(TMPL,">$file") ||
	croak("Cannot write to $file: $!");

    # generate the output file
    print TMPL $template->output();
    close(TMPL);

    # output file was written successfully
    return 1;
#**********************************************************************#
# END SUBROUTINE:                         #
#**********************************************************************#
}

sub unlock_datafile {
#**********************************************************************#
# SYNTAX:                                  #
# PARAMS:                  #
# RETURN: 1 -                                   #
#         0 -                                    #
#**********************************************************************#
    # grab the new link parameters
    my ($self,$file) = @_;
  
    # close the semaphore file and delete it
    close(SEM);
    unlink "$file.semaphore";
#**********************************************************************#
# END SUBROUTINE:                         #
#**********************************************************************#
}


1;
